/** TransmitIRSignal: Takes raw IR time data over serial and transmits it on pin 11

 *  Can take up to 192 signal transitions (48 bits). Uses a 38 KHz carrier generated
 *  by directly setting timer 2's registers.
 *  Author: Tom Lauwers, tlauwers@birdbraintechnologies.com
 *  Date: 12/22/2011
 */

unsigned int incoming_times[128]; // Contains parsed time data
byte incoming_bytes[384]; // Buffer for incoming serial data
unsigned int temp[7]; // Holds temporary data for parsing
int i = 0; // Loop counter
int k = 0; // Another loop counter
int countBytes = 0; // Counts incoming bytes
boolean toggleLED = true; // Flag to determine if we need to turn LED on or off
boolean validTimes = true; // Flag to determine if incoming serial stream contained valid data
int multiplier = 1; // Used for raising 10 to the power of x (so we don't use Math's broken pow function)
int timeout = 0;  // Times out serial if we hear nothing after a while

void setup()
{
  // start serial port at 57600 bps:
  Serial.begin(57600);
  
  // PWM Magic - we directly set atmega registers to 50% duty cycle,
  // variable frequency mode. Currently set to 38 KHz.
  // Thanks http://arduino.cc/en/Tutorial/SecretsOfArduinoPWM
  TCCR2A = _BV(WGM21) | _BV(WGM20);
  //set prescalar to 16MHz clock for now
  TCCR2B = _BV(WGM22) | _BV(CS20);
  // To get a 38KHz wave, we need OCR2A = (16000000/76000)-1 = 209
  OCR2A = 209; // OCRA2A = 140 for 56KHz operation
  // Set the pin the IR LED is on to output
  pinMode(11, OUTPUT);
  
  
}

void loop()
{
  // Reset our counters
  i=0;
  k=0;
  timeout = 0;
  countBytes=0;
  
  // if we get a valid byte, start transmitting data
  if (Serial.available() > 0) {
    while(timeout < 30000) {
      // Read data as fast as you can (every 10 us)
      temp[0] = Serial.read();
      // If there's actually data, put it in the incoming_bytes buffer
      if(temp[0] != -1) {
        timeout = 0; // Reset timeout
        incoming_bytes[countBytes] = (byte)temp[0];
        countBytes++;
      }
      timeout++;
      delayMicroseconds(10);
    }
    // Once you have the bytes, parse them into times
    for(int m = 0; m < countBytes; m++) { 
      validTimes = true;
      temp[k] = incoming_bytes[m]-'0'; // Hacky way to turn ascii into numeric
      k++;
      // If we saw a comma, that's the end of one number
      if(temp[k-1] == (','-'0')) 
      {
        for(int j = k-2; j >=0; j--) { 
          /* Hack because power function doesn't calculate well - according to it 10^2 = 99.9 instead of 100*/
          multiplier = 1;
          for(int l = 0; l < k-2-j; l++)
          {
            multiplier *= 10;
          }
          incoming_times[i] += temp[j] * multiplier; 
        }
        // No IR signal should have a time of 99us or less
        if(incoming_times[i] < 100) {
          Serial.print("Invalid signal time - too short\n");
          Serial.flush();
          validTimes = false;
          break;
        }
        k = 0;
        Serial.print(incoming_times[i]);
        Serial.print("us ");
        i++;
      }
    }
    // All streams should be odd (first and last times specified are both signal high)
    if(i%2 != 1) {
          Serial.print("Invalid - even number of signals\n");
          Serial.flush();
          validTimes = false;      
    }
  
    Serial.flush();
    // If the data was correctly constructed, transmit it
    if(validTimes)
    {
      Serial.print("\nSignal received, now transmitting\n");
      // Loop through all the signal times
      for(int j = 0; j < i; j++)
      {
        // Turn the LED on or off (first time through LED is turned on)
         if(toggleLED) {
           // Connect pin to signal, this turns on the LED
           TCCR2A |=  _BV(COM2A0);
           toggleLED = false;
         } 
         else {
           TCCR2A &= (0xFF - _BV(COM2A0)); // disconnect pin from signal
           digitalWrite(11,LOW); // turn pin low in case it's high
           toggleLED = true;
         } 
         // Delays by the time requested
         delayMicroseconds(incoming_times[j]); 
      }
      // Turn off the signal after we end the loop
      TCCR2A &= (0xFF - _BV(COM2A0)); // disconnect pin from signal
      digitalWrite(11,LOW); // turn pin low in case it's high
      toggleLED = true;
      Serial.print("Transmission complete\n");
    }
    // Reset the array (important if next signal stream is shorter)
    for(int j = 0; j < i; j++) {
      incoming_times[j] = 0;
    }
  }
}

